<?php

namespace Drupal\quick_data\Hook;

use Drupal\commerce_order\Entity\OrderItemType;
use Drupal\commerce_order\Entity\OrderType;
use Drupal\commerce_product\Entity\ProductType;
use Drupal\commerce_store\Entity\StoreType;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\filter\FilterPluginManager;

/**
 * Hook implementations for ocr image.
 */
class QuickDataHooks {
  use StringTranslationTrait;

  /**
   * Constructs a new DataFieldHook object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The message service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   The entity type bundle info service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Extension\ModuleExtensionList|null $moduleExtensionList
   *   The module list service.
   * @param \Drupal\filter\FilterPluginManager|null $filterPluginManager
   *   Filter plugin manager.
   */
  public function __construct(
    protected ModuleHandlerInterface $moduleHandler,
    protected ConfigFactoryInterface $configFactory,
    protected MessengerInterface $messenger,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ?ModuleExtensionList $moduleExtensionList = NULL,
    protected ?FilterPluginManager $filterPluginManager = NULL,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_type.manager'),
      $container->get('extension.list.module'),
      $container->get('plugin.manager.filter'),
    );
  }

  /**
   * Implements hook_help().
   */
  #[Hook('help')]
  public function help($route_name, RouteMatchInterface $route_match) {
    switch ($route_name) {
      // Main module help for the quick_data module.
      case 'help.page.quick_data':

        $entity_types = $this->entityTypeManager->getDefinitions();
        $fieldable_entity_types = array_filter($entity_types, function ($entity_type) {
          return method_exists($entity_type->getOriginalClass(), 'hasField');
        });
        $entityTypeNotReadable = [
          'file',
          'menu_link_content',
          'path_alias',
          'shortcut',
        ];
        $entity_types = [];
        foreach ($fieldable_entity_types as $type => $entity_type) {
          if ($type == 'paragraph') {
            continue;
          }
          if (!in_array($type, $entityTypeNotReadable)) {
            $entity_types[$type] = [
              '#type' => 'details',
              '#title' => $this->t('Example: %name - %label', [
                '%name' => $type,
                '%label' => $entity_type->getLabel(),
              ]),
              '#open' => TRUE,
              'content' => [
                '#theme' => 'item_list',
                '#title' => $this->t('List link import'),
                '#items' => [],
                '#attributes' => [
                  'class' => ['custom-list-class'],
                ],
              ],
            ];
            $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($type);
            foreach ($bundle_info as $key => $name) {
              $entity_types[$type]['content']['#items'][] = [
                '#type' => 'link',
                '#title' => $name['label'] ?? $key,
                '#url' => Url::fromRoute('quick_data.content.import', [
                  'type' => $type,
                  'bundle' => $key,
                ]),
              ];
            }
          }
        }

        if ($this->moduleHandler->moduleExists('commerce')) {
          // @phpstan-ignore-next-line
          $store_types = StoreType::loadMultiple();
          $type = 'commerce';
          $entity_types[$type]['label'] = $this->t('Ecommerce');
          foreach ($store_types as $store_type) {
            $entity_types[$type]['bundles']['commerce_store/' . $store_type->id()] = $this->t("Store") . " - " . $store_type->label();
          }
          // @phpstan-ignore-next-line
          $product_types = ProductType::loadMultiple();
          foreach ($product_types as $product_type) {
            $entity_types[$type]['bundles']['commerce_product/' . $product_type->id()] = $this->t("Product") . " - " . $product_type->label();
          }
          // @phpstan-ignore-next-line
          $order_items_types = OrderItemType::loadMultiple();
          foreach ($order_items_types as $order_items_type) {
            $entity_types[$type]['bundles']['commerce_order_item/' . $order_items_type->id()] = $this->t("Order items") . " - " . $order_items_type->label();
          }
          // @phpstan-ignore-next-line
          $order_types = OrderType::loadMultiple();
          foreach ($order_types as $order_type) {
            $entity_types[$type]['bundles']['commerce_order/' . $order_type->id()] = $this->t("Order") . " - " . $order_type->label();
          }
        }
        include_once \Drupal::root() . '/core/themes/engines/twig/twig.engine';
        $path = $this->moduleExtensionList->getPath('quick_data');
        if (function_exists('twig_render_template')) {
          return twig_render_template($path . '/templates/help-body.html.twig', [
            'entity_types' => $entity_types,
            'theme_hook_original' => 'not-applicable',
            'directory' => $path,
          ]);
        }

      default:
        break;
    }
  }

}
